On 01/04/15 15:14, Tapani Pälli wrote:
Use program_resource_location to fetch location.

Signed-off-by: Tapani Pälli <tapani.pa...@intel.com>
---
  src/mesa/main/shader_query.cpp | 38 ++++++++++++++++----------------------
  1 file changed, 16 insertions(+), 22 deletions(-)

diff --git a/src/mesa/main/shader_query.cpp b/src/mesa/main/shader_query.cpp
index cbdcc61..58f5a0e 100644
--- a/src/mesa/main/shader_query.cpp
+++ b/src/mesa/main/shader_query.cpp
@@ -484,31 +484,25 @@ _mesa_GetFragDataLocation(GLuint program, const GLchar 
*name)
     if (shProg->_LinkedShaders[MESA_SHADER_FRAGMENT] == NULL)
        return -1;
- exec_list *ir = shProg->_LinkedShaders[MESA_SHADER_FRAGMENT]->ir;
-   foreach_in_list(ir_instruction, node, ir) {
-      const ir_variable *const var = node->as_variable();
-
-      /* The extra check against FRAG_RESULT_DATA0 is because
-       * glGetFragDataLocation cannot be used on "conventional" attributes.
-       *
-       * From page 95 of the OpenGL 3.0 spec:
-       *
-       *     "If name is not an active attribute, if name is a conventional
-       *     attribute, or if an error occurs, -1 will be returned."
-       */
-      if (var == NULL
-         || var->data.mode != ir_var_shader_out
-         || var->data.location == -1
-         || var->data.location < FRAG_RESULT_DATA0)
-        continue;
+   struct gl_program_resource *res =
+      _mesa_program_resource_find_name(shProg, GL_PROGRAM_OUTPUT, name);
- int index = get_matching_index(var, (const char *) name);
+   if (!res)
+      return -1;
- if (index >= 0)
-         return var->data.location + index - FRAG_RESULT_DATA0;
-   }
+   GLint loc = program_resource_location(shProg, res, name);
- return -1;
+   /* The extra check against against 0 is made because of builtin-attribute
+    * locations that have offset applied. Function program_resource_location
+    * can return built-in attribute locations < 0 and glGetFragDataLocation
+    * cannot be used on "conventional" attributes.
+    *
+    * From page 95 of the OpenGL 3.0 spec:
+    *
+    *     "If name is not an active attribute, if name is a conventional
+    *     attribute, or if an error occurs, -1 will be returned."
+    */
+   return (loc >= 0) ? loc : -1;
  }
const char*
Great code-deduplication again.

Reviewed-by: Martin Peres <martin.pe...@linux.intel.com>
_______________________________________________
mesa-dev mailing list
mesa-dev@lists.freedesktop.org
http://lists.freedesktop.org/mailman/listinfo/mesa-dev

Reply via email to